; Forever Tunnel v1.1 for TSE
; By Robin Kay
;

; You may not distribute a modified version of this program. However, you
; may take code from or base your program on this code providing that the
; resulting program is at least 50% of your own work.

.nolist
#include "tse.inc"
.list
.org    usermem-3
.db     $BB,$6D
 ret

.db "TSE",1              ; header
prgtitle:
.db " Forever Tunnel", 0 ; title
.dw 88                   ; stack required

width .equ 8

start:

 ; Dispaly main menu
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl
 ld hl, prgtitle + 1
 call putStr
 ld hl, author
 call putStr
 bcall(_newline)
 ld hl, txt1
 call putStr
 ld hl, txt2
 call putStr
 bcall(_newline)
 bcall(_newline)
 ld hl, hstxt
 bcall(_puts)
 ld hl, (highscore)
 bcall(_disphl)

 ; Poll for keypad on menu
loop:
 ei
 halt ; Save power
 bcall(_getcsc)

 cp sk1
 jr nz, cont0
 ld h, 10
 jr maingame
cont0:

 cp sk2
 jr nz, cont1
 ld h, 5
 jr maingame
cont1:

 cp sk3
 jr nz, cont2
 ld h, 2
 jr maingame
cont2:

 ; Process other keys
 cp skclear
 jr z, clrhs

 ; Yield
 cp skStat
 jr nz, dontyield
 call _tseforceyield
 jr start
dontyield:
 jr loop

; Clear highscore
clrhs:
 ld hl, 0
 ld (highscore), hl
 jr start 

; Main game routine
maingame:

 ; Save delay
 ld l, 0
 ld (delay), hl

 ; Initalise the tunnel map
 ld hl, tunnel_map
 ld bc, 256*32+(48-((width+32+8)/2))
setupmap:
 ld (hl), c
 inc hl
 djnz setupmap

 ; Initalise the car position, tunnel direction, and score
 ld a, 48-4
 ld (car_pos), a
 xor a
 ld (tunnel_dir), a
 ld hl, 0
 ld (score), hl

redrawgame:

 ; Clear the graph buffer
 bcall(_GrBufClr)

 ; Init redraw display
 ld hl, tunnel_map
 ld b, 32

loopdisp:
 push bc
 push hl

 ; Displey left wall
 ld a, (hl)
 ld c, a
 push bc
 ld l, b
 dec l
 sla l
 ld b, 2
 ld ix, wall
 call ionputsprite
 pop bc

 ; Display right wall
 ld a, c
 add a, 32+width
 ld l, b
 dec l
 sla l
 ld b, 2
 call ionputsprite

 ; Move to next tunnel segment
 pop hl
 inc hl
 pop bc
 djnz loopdisp

 ; Draw onto sceen
 call ionfastcopy

loopgame:

 ; Delay
 ld bc, (delay)
 ld hl, 8000h
 ld d, h
 ld e, l
 ldir ; Nop!

 ; Increase score
 ld hl, (score)
 inc hl
 ld (score), hl

 ; Scroll data
 ld hl, tunnel_map + 1
 ld de, tunnel_map
 ld bc, 31
 ldir

 ; Change direction
again:
 ld b, 5
 call ionrandom
 cp 0
 jr nz, dontchange
zero:
 ld b, 3
 call ionrandom
 cp 1
 jr z, zero
 dec a
 ld (tunnel_dir), a
dontchange:

 ; Calculate new wall segment position
 ld a, (top_tunnel)
 ld b, a
 ld a, (tunnel_dir)
 add a, b

 ; Validity check
 cp 96-(width+32+width)
 jr c, ok
 ld a, b
ok:

 ; Write data back
 ld (top_tunnel), a

 ; Scroll display
 ld de,plotsscreen+767
 ld hl,plotsscreen+767-24
 ld bc,768-24
 lddr

 ; Clear top two lines
 ld b, 24
 ld hl, plotsscreen
 xor a
clrloop:
 ld (hl), a
 inc hl
 djnz clrloop

 ; Draw new left wall segment
 ld a, (top_tunnel)
 ld l, 0
 ld b, 2
 ld ix, wall
 call ionputsprite

 ; Draw new right wall segment
 ld a, (top_tunnel)
 add a, 32+width
 ld l, 0
 ld b, 2
 call ionputsprite

 ; Process keypad
 ld a,kreset
 out (1),a
 ld a,group1
 out (1),a
 in a,(1)
 cp kright
 jr z, right
 cp kleft
 jr nz, nothing
left:
 ld a, (car_pos)
 dec a
 ld (car_pos), a
 jr nothing
right:
 ld a, (car_pos)
 inc a
 ld (car_pos), a
nothing:

 ; Draw car
 call xorCar

 ; Redraw display
 call ionfastcopy

 ; Clear car
 call xorCar

 ; Check for left collision
 ld a, (tunnel_map)
 add a, 8
 ld b, a
 ld a, (car_pos)
 cp b
 jr c, done

 ; Check for right collision
 ld a, (tunnel_map)
 add a, (32+width)-8
 ld b, a
 ld a, (car_pos)
 cp b
 jr nc, done

 ; Yield
 ld a,kreset
 out (1),a
 ld a,group4
 out (1),a
 in a,(1)
 cp KStat
 jp nz, loopgame
 call _tseforceyield
 jp redrawgame

; Hiscore screen
done:
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl
 ld hl, scrtxt
 bcall(_puts)
 ld hl, (score)
 bcall(_disphl)
 bcall(_newline)
 ld hl, (score)
 ld de, (highscore)
 inc de
 bcall(_cphlde)
 jr c, nohighscore

 ld (highscore), hl
 ld hl, newhs
 bcall(_puts)

nohighscore:
 ei
 halt ; Save power
 bcall(_getcsc)
 cp skMode
 jp z, start
 cp skStat
 jr nz, nohighscore
 call z, _tseforceyield
 jr done

; Display string and then advance a line onwards
putStr:
 bcall(_puts)
 bcall(_newline)
 ret

; XOR the car onto the screen
xorCar:
 ld a, (car_pos)
 ld l, 64-5
 ld b, 5
 ld ix, car
 call ionputsprite
 ret

highscore:
.dw 0

wall:
.db %11111111 
.db %11111111
.db %11111111 ; Second wall to save writting to IX twice
.db %11111111

car:
.db %00011000
.db %00111100
.db %01111110
.db %11000011
.db %10000001

author:
.db "By Robin Kay", 0
txt1:
.db "1-3: Start Game",0
txt2:
.db "Clr: Reset HS",0
hstxt:
.db "Hi-"
scrtxt
.db "Score: ",0
newhs:
.db "New HighScore!!",0

dta:
tunnel_map      .equ dta
top_tunnel      .equ tunnel_map + 31
tunnel_dir      .equ top_tunnel + 1
car_pos         .equ tunnel_dir + 1
score           .equ car_pos + 1
delay           .equ score + 2

.end
